type t

type lemma = {
  vars: Symbols.Set.t;  (* toplevel quantified variables *)
  trs : Term.t list list; (* multi-triggers *)
  f : t;  (* the main lemma's formula *)
  name : string
}

and view = 
    U of t list   (* unit clauses *)
  | C of t*t      (* a clause (t1 or t2) *)
  | A of Atom.t   (* an atom *)
  | L of lemma    (* a lemma *)
  | Sko of Term.t Subst.t *  t  (* lazy substitution *)

val mk_not : t -> t
val mk_and : t -> t -> t
val mk_or : t -> t -> t
val mk_imp : t -> t -> t
val mk_if : t -> t -> t -> t
val mk_iff : t -> t -> t
val mk_atom : Atom.t -> t
val mk_forall : 
 Term.Set.t -> Term.Set.t -> Term.Set.t -> Term.t list list -> t -> string -> t
val mk_exists : 
 Term.Set.t -> Term.Set.t -> Term.Set.t -> Term.t list list -> t -> string -> t

val view : t -> view
val print : Format.formatter -> t -> unit

val terms : t -> Term.Set.t
val free_vars : t -> Symbols.Set.t

val apply_subst : Term.subst -> t -> t 

val compare : t -> t -> int
    
module Set : Set.S with type elt = t
module Map : Map.S with type key = t

