open Format
open Options

module T = Term
module F = Formula

type gsubst = { sbt : T.subst ; gen : int }

module type X = sig
  type t

  val classof : t -> Term.t -> Term.t list
  val query : Atom.t -> t -> bool

end

module type S = sig
  type t
  type uf

  val empty : t
  val add : int -> F.t option-> Term.t -> t -> t 
  val matching : F.t -> Term.t list -> t -> uf -> gsubst list
end

module Make (X : X) = struct

  type uf = X.t

  module MT = T.Map

  type info = {age : int ; lem_orig : F.t option }
  type t = { fils : T.t list MT.t Subst.t ; info : info MT.t}

  exception Echec
    
  module SubstT = Subst.Make(T)

  let empty = { fils = SubstT.empty ; info = MT.empty }

  let age_limite = 3

  let new_age op t g env = 
    try op (MT.find t env.info).age g with Not_found -> g

  let add age lem t env =
    let rec add_rec env t = 
      let {T.f=f;xs=xs} = T.view t in
      let env = 
	let map_f = try SubstT.find f env.fils with Not_found -> MT.empty in
	{fils = SubstT.add f (MT.add t xs map_f) env.fils; 
	 info=MT.add t {age=(new_age min t age env); lem_orig=lem} env.info}
      in
      List.fold_left add_rec env xs
    in
    if astuce then 
      if age>age_limite then env else add_rec env t
    else add_rec env t
      
  exception Deja_vu
  let deja_vu l = function None -> false | Some l' -> F.compare l l'=0

  let all_terms f ty env lem {sbt=(s_t,s_ty);gen=g} = 
    SubstT.fold 
      (fun k s l-> 
	 MT.fold 
	   (fun t _ l -> 
	      try
		let s_ty = Ty.matching s_ty ty (T.view t).T.ty in
		let ng = 
		  try 
		    let {age=ng;lem_orig=lem'} = MT.find t env.info in
		    if deja_vu lem lem' then raise Deja_vu;
		    max ng g 
		  with Not_found -> g
		in
		{sbt=(SubstT.add f t s_t, s_ty);gen=ng}::l 
	      with Ty.TypeClash _ | Deja_vu-> l
	   )
	   s l)
      env.fils []

  let add_msymb uf f t ({sbt=(s_t,s_ty);gen=g} as sg)= 
    try 
      let t' = SubstT.find f s_t in
      if X.query (Atom.make(Atom.Eq(t,t'))) uf then sg 
      else raise Echec
    with Not_found ->  {sg with sbt=(SubstT.add f t s_t,s_ty) }

  let rec iter_exception f m = function
      [] -> raise Echec
    | x::l -> try f m x with Echec -> iter_exception f m l
      
  let rec matchterm env uf ( {sbt=(s_t,s_ty);gen=g} as sg) pat t =
    let {T.f=f_pat;xs=pats;ty=ty_pat} =  T.view pat in
    match f_pat with
	Symbols.Var _ -> 
	  (try
	     let s_ty = Ty.matching s_ty ty_pat (T.view t).T.ty in
	     add_msymb uf f_pat t {sbt=(s_t,s_ty);gen=new_age max t g env}
	   with Ty.TypeClash _ -> raise Echec)
      | _ -> 
	  let l = List.map T.view (X.classof uf t) in
	  let s_ty , l = 
	    List.fold_left
	      (fun (s_ty,l) ({T.f=f;ty=ty_t} as t) -> 
		 if Symbols.compare f_pat f=0 then 
		   try
		     let s_ty = Ty.matching s_ty ty_pat ty_t in
		     s_ty , t::l 
		   with Ty.TypeClash _ -> s_ty , l
		 else s_ty , l
	      ) (s_ty,[]) l 
	  in
	  iter_exception (* pas sur que ce soit correct ici *)
	    (fun m {T.xs=xs} -> matchterms env uf m pats xs) 
	    { sg with sbt = (s_t,s_ty)} l
	  
  and matchterms env uf sg pats xs = 
    try List.fold_left2 (matchterm env uf) sg pats xs
    with Invalid_argument _ -> raise Echec

  let matchpat env uf lem lsubst ({gen=g} as sg,pat) = 
    let {T.f=f;xs=pats;ty=ty} = T.view pat in
    match f with
	Symbols.Var _ -> all_terms f ty env lem sg
      | _ -> 
	  try  MT.fold 
	    (fun t xs l -> try 
	       (matchterms env uf 
		  {sg with gen=new_age max t g env} pats xs)::l 
	     with Echec -> l)
	    (SubstT.find f env.fils) lsubst
	  with Not_found -> lsubst
	    
  let matchpats env uf lem lsubsts pat = 
    let lpats = 
      List.map (fun sg -> (sg,T.apply_subst sg.sbt pat)) lsubsts in
    List.flatten (List.map (matchpat env uf lem []) lpats)

  let matching lem pats env uf = 
    let egs = {sbt=(SubstT.empty,Ty.esubst) ; gen = 0} in
    List.fold_left (matchpats env uf lem) [egs] pats

end
