module type COEF = sig
  type t
  val of_int : string -> t
  val zero : t
  val one : t 
  val minusone : t 
  val plus : t -> t -> t
  val minus : t -> t -> t
  val mult : t -> t -> t
  val inverse : t -> t

  val compare : t -> t -> int

  val print : Format.formatter -> t -> unit

end

module type S = sig

  type coef

  exception Unsolvable

  type dumview = coef Term.Map.t
  type view = coef Term.Map.t * coef
  type t

  module Set : Set.S with type elt = t
  module Map : Map.S with type key = t

  val is_num : t -> bool
  val is_var : t -> Term.t Sig.mine
  
  val make : Term.t -> t

  val leaves : t -> Term.Set.t

  val view : t -> view
  val dumview : t -> dumview

  val compare : t -> t -> int

  val mult : coef -> t -> t
  val add : t -> t -> t

  val choose : t -> Term.t * coef
  val subst : Term.t -> t -> t -> t
  val canon : t -> (Term.t -> t) -> t 
  val solve : t -> t -> (Term.t * t) list

  val print : Format.formatter -> t -> unit

end

module Make (C:COEF) : S with type coef=C.t 
